/*
 * Filter.c
 * 
 * Array filtering functions for JohnnyFive.ino
 */

/******************************************************************************
 * Variables and constants
 *****************************************************************************/

const int FIELD_MINIMUM = 6; //Filter out closer than six inches
//Max distances to stay within our 10' by 10' field for each degree of scanning;
const int fieldMaxArray[180] = {60, 60, 60, 60, 60, 60, 60, 60, 61, 61, 
                            61, 61, 61, 62, 62, 62, 62, 63, 63, 63, 
                            64, 64, 65, 65, 66, 66, 67, 67, 68, 69, 
                            69, 70, 71, 72, 72, 73, 74, 75, 76, 77, 
                            78, 80, 81, 82, 83, 85, 86, 88, 90, 91, 
                            93, 95, 97, 100, 102, 105, 107, 110, 113, 116, 
                            120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 
                            120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 
                            120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 
                            120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 
                            120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 
                            120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 
                            120, 116, 113, 110, 107, 105, 102, 100, 97, 95, 
                            93, 91, 90, 88, 86, 85, 83, 82, 81, 80, 
                            78, 77, 76, 75, 74, 73, 72, 72, 71, 70, 
                            69, 69, 68, 67, 67, 66, 66, 65, 65, 64, 
                            64, 63, 63, 63, 62, 62, 62, 62, 61, 61, 
                            61, 61, 61, 60, 60, 60, 60, 60, 60, 60 };
                            
const int CALIBRATION_TARGET_THRESHOLD = 20;


/******************************************************************************
 * Prototypes and Comments
 *****************************************************************************/

/** 
 * Loop through the array, replacing out bad values with zeros.
 * 
 * @param array[] the array to operate on.  Will change the original because arrays are passed by pointer.
 * @param length   the length of the array
 * @param high     the high-bound for the filter
 * @param low      the low bound for the filter
 */
void filterData(int array[], int length, int high, int low);

/*
 * Loop through an array[180] and filter out distances too great to reach outside
 *  5 feet to either side and 10 feet forward.
 * IMPORTANT: ONLY WORKS ON A HORIZONTAL (yaw) SCAN OF 180.
 */
void fieldFilterData(int array[], int length);

/*
 * Loop through an array and remove a portion of the echos from
 *  the calibration targets located at 180deg, 2ft and 0deg, 3 feet.
 */
void filterCalibrationTargets(int array[], int length);

/**
 * Loop through the array, replacing each value with an average of it and it's neighbors.
 * This will replace up to a single zero inside a string of good values.
 */
void smoothFilterData(int array[], int length);

/******************************************************************************
 * Function Definitions
 *****************************************************************************/

void filterData(int array[], int length, int high, int low)
{
  int i; //declaration in for() Not allowed prior to C99
  for(i = 0; i < length; i++){
    if(array[i] > high || array[i] < low){
      array[i] = 0;
    }
  }
}

void fieldFilterData(int array[], int length)
{
  int i;
  for(i = 0; i < length; i++){
    if(array[i] > (fieldMaxArray[i] + 6) || array[i] < 6){
      array[i] = 0;
    }
  }
}

void filterCalibrationTargets(int array[], int length)
{
  int i;
  int j;
  for(i = 0; i < CALIBRATION_TARGET_THRESHOLD; i++){ //Filter out the three-foot (36in) target
    if( (array[i] - 36) <= 6){//if within 6 inches of 36
      array[i] = 0;
    }
  }
  for(j = length; j > length - CALIBRATION_TARGET_THRESHOLD; j--){ //Filter out the two-foot (24in) target
    if( (array[j] - 24) <= 6){//if within 6 inches of 24
      array[j] = 0;
    }
  }
}

void smoothFilterData(int array[], int length)
{
  int newArray[length];
  int i; //declaration in for() Not allowed prior to C99
  for(i = 0; i < length; i++){
    //Don't do the three-value average on the first and last elements, or we'll be reading out of bounds
    if(i == 0 || i == length - 1){
      newArray[i] = array[i];
    }
    else{
      newArray[i] = (array[i-1] + array[i] + array[i+1]) / 3;//Average of left-neighbor, i, and right neighbor
    }
  }
  //Replace the data in the original array with the data in the new one
  int j;
  for(j = 0; j < length; j++){
    array[j] = newArray[j];
  }
}
